==> pybrv-0.1.7-py3-none-any.whl: pybrv/recommendation.py <==
# pybrv_recommendation.py

from openai import OpenAI
from pyspark.sql.functions import col, countDistinct, count, min, max
from typing import List
from pyspark.sql import SparkSession
import os

class pybrv_recommendation:
    def __init__(self):
        self.DATABRICKS_TOKEN = dbutils.notebook.entry_point.getDbutils().notebook().getContext().apiToken().get()
        self.client = OpenAI(
            api_key=self.DATABRICKS_TOKEN,
            base_url="https://adb-7837506776017759.19.azuredatabricks.net/serving-endpoints"
        )

    def detect_relationships(self, dfs: dict) -> list:
        """Detect potential foreign key relationships between tables."""
        relationships = []
        for src_table, src_df in dfs.items():
            for src_col in src_df.columns:
                for tgt_table, tgt_df in dfs.items():
                    if src_table == tgt_table:
                        continue
                    if src_col in tgt_df.columns:
                        src_unique = src_df.select(src_col).distinct().count()
                        join_match = src_df.join(tgt_df, on=src_col, how="left_anti").count()
                        if src_unique > 1 and join_match == 0:
                            relationships.append((src_table, src_col, tgt_table, src_col))
        return relationships

    def analyze_multiple_tables(self, table_names: List[str]) -> str:
        dfs = {table: spark.table(table) for table in table_names}
        report = []

        for table_name, df in dfs.items():
            report.append(f"\n=== Table: `{table_name}` ===")
            for col_name, dtype in df.dtypes:
                stats = df.select(
                    count("*").alias("total_rows"),
                    count(col(col_name)).alias("non_nulls"),
                    countDistinct(col(col_name)).alias("unique_values"),
                    min(col(col_name)).alias("min_val"),
                    max(col(col_name)).alias("max_val")
                ).first()

                report.append(f"""
  Column: `{col_name}`
    - Type: {dtype}
    - Non-null Count: {stats['non_nulls']}
    - Unique Values: {stats['unique_values']}
    - Min: {stats['min_val']}
    - Max: {stats['max_val']}
                """)

        relationships = self.detect_relationships(dfs)
        report.append("\n=== Detected Relationships (FK-like) ===")
        if relationships:
            for src_table, src_col, tgt_table, tgt_col in relationships:
                report.append(f"- `{src_table}.{src_col}` → `{tgt_table}.{tgt_col}` (Valid)")
        else:
            report.append("No relationships detected.")

        return "\n".join(report)

    def get_test_cases_for_tables(self, table_names: list, metadata: str) -> str:
        prompt = f"""
        You are a data quality expert.
        Your task is to review the metadata of the tables `{', '.join(table_names)}` and generate data quality test cases as SQL queries.
        Suggest test cases to validate data integrity, consistency, and relationships.

        ### Requirements:
        - Give the pass sql and fail sql for each test case.
        - Specify severity (low, medium, high, critical).
        - Cover: Null checks, Uniqueness checks, Completeness, Duplicates, Accuracy, Validation.
        - Provide a JSON config for each test case with:
            "RULE_ID", "RULE_NAME", "FAIL_SQL", "PASS_SQL", "SEVERITY", "RULE_CATEGORY", "TABLES_CHECKED", "INVENTORY", "COMMENTS", "PASS_THRESHOLD"

        ### Table Metadata:
        {', '.join(table_names)}\n\n{metadata}
        """

        response = self.client.chat.completions.create(
            model="databricks-claude-3-7-sonnet",
            messages=[{"role": "user", "content": prompt}]
        )
        return response.choices[0].message.content

    def suggest_tests(self, table_names: List[str]):
        print("🔍 Analyzing table...")
        metadata = self.analyze_multiple_tables(table_names)
        print(metadata)

        print("\n🧠 Generating test cases using AI model...\n")
        test_cases = self.get_test_cases_for_tables(table_names, metadata)
        print(test_cases)

==> pybrv-0.1.9-py3-none-any.whl: pybrv/recommendation.py <==
# pybrv_recommendation.py

from openai import OpenAI
from pyspark.sql.functions import col, countDistinct, count, min, max
from typing import List
from pyspark.dbutils import DBUtils
from pyspark.sql import SparkSession
import os

# class pybrv_recommendation:
#     def __init__(self,dbutils,spark: SparkSession):
#         self.spark = spark.getActiveSession()
#         self.DATABRICKS_TOKEN = dbutils.notebook.entry_point.getDbutils().notebook().getContext().apiToken().get()
#         self.client = OpenAI(
#             api_key=self.DATABRICKS_TOKEN,
#             base_url="https://adb-7837506776017759.19.azuredatabricks.net/serving-endpoints"
#         )
class pybrv_recommendation:
    def __init__(self, spark: SparkSession, dbutils):
        self.spark = spark.getActiveSession()
        self.dbutils = dbutils
        self.client = OpenAI(
            api_key=self.DATABRICKS_TOKEN,
            base_url="https://adb-7837506776017759.19.azuredatabricks.net/serving-endpoints"
        )

    def detect_relationships(self, dfs: dict) -> list:
        """Detect potential foreign key relationships between tables."""
        relationships = []
        for src_table, src_df in dfs.items():
            for src_col in src_df.columns:
                for tgt_table, tgt_df in dfs.items():
                    if src_table == tgt_table:
                        continue
                    if src_col in tgt_df.columns:
                        src_unique = src_df.select(src_col).distinct().count()
                        join_match = src_df.join(tgt_df, on=src_col, how="left_anti").count()
                        if src_unique > 1 and join_match == 0:
                            relationships.append((src_table, src_col, tgt_table, src_col))
        return relationships

    def analyze_multiple_tables(self, table_names: List[str]) -> str:
        dfs = {table: self.spark.table(table) for table in table_names}
        report = []

        for table_name, df in dfs.items():
            report.append(f"\n=== Table: `{table_name}` ===")
            for col_name, dtype in df.dtypes:
                stats = df.select(
                    count("*").alias("total_rows"),
                    count(col(col_name)).alias("non_nulls"),
                    countDistinct(col(col_name)).alias("unique_values"),
                    min(col(col_name)).alias("min_val"),
                    max(col(col_name)).alias("max_val")
                ).first()

                report.append(f"""
  Column: `{col_name}`
    - Type: {dtype}
    - Non-null Count: {stats['non_nulls']}
    - Unique Values: {stats['unique_values']}
    - Min: {stats['min_val']}
    - Max: {stats['max_val']}
                """)

        relationships = self.detect_relationships(dfs)
        report.append("\n=== Detected Relationships (FK-like) ===")
        if relationships:
            for src_table, src_col, tgt_table, tgt_col in relationships:
                report.append(f"- `{src_table}.{src_col}` → `{tgt_table}.{tgt_col}` (Valid)")
        else:
            report.append("No relationships detected.")

        return "\n".join(report)

    def get_test_cases_for_tables(self, table_names: list, metadata: str) -> str:
        prompt = f"""
        You are a data quality expert.
        Your task is to review the metadata of the tables `{', '.join(table_names)}` and generate data quality test cases as SQL queries.
        Suggest test cases to validate data integrity, consistency, and relationships.

        ### Requirements:
        - Give the pass sql and fail sql for each test case.
        - Specify severity (low, medium, high, critical).
        - Cover: Null checks, Uniqueness checks, Completeness, Duplicates, Accuracy, Validation.
        - Provide a JSON config for each test case with:
            "RULE_ID", "RULE_NAME", "FAIL_SQL", "PASS_SQL", "SEVERITY", "RULE_CATEGORY", "TABLES_CHECKED", "INVENTORY", "COMMENTS", "PASS_THRESHOLD"

        ### Table Metadata:
        {', '.join(table_names)}\n\n{metadata}
        """

        response = self.client.chat.completions.create(
            model="databricks-claude-3-7-sonnet",
            messages=[{"role": "user", "content": prompt}]
        )
        return response.choices[0].message.content

    def suggest_tests(self, table_names: List[str]):
        print("🔍 Analyzing table...")
        metadata = self.analyze_multiple_tables(table_names)
        print(metadata)

        print("\n🧠 Generating test cases using AI model...\n")
        test_cases = self.get_test_cases_for_tables(table_names, metadata)
        print(test_cases)
